@@ -4,8 +4,12 @@ use compat::{es2015::regenerator, es2020::export_namespace_from};
 use either::Either;
 use swc_atoms::JsWord;
 use swc_common::{
-    chain, comments::Comments, errors::Handler, sync::Lrc, util::take::Take, FileName, Mark,
-    SourceMap,
+    chain,
+    comments::{Comments, SingleThreadedComments},
+    errors::Handler,
+    sync::Lrc,
+    util::take::Take,
+    FileName, Mark, SourceMap,
 };
 use swc_ecma_ast::{EsVersion, Module};
 use swc_ecma_minifier::option::MinifyOptions;
@@ -17,9 +21,8 @@ use swc_ecma_transforms::{
 };
 use swc_ecma_visit::{as_folder, noop_visit_mut_type, VisitMut};
 
-use crate::{
-    config::{util::BoolOrObject, CompiledPaths, GlobalPassOption, JsMinifyOptions, ModuleConfig},
-    SwcComments,
+use crate::config::{
+    util::BoolOrObject, CompiledPaths, GlobalPassOption, JsMinifyOptions, ModuleConfig,
 };
 
 /// Builder is used to create a high performance `Compiler`.
@@ -162,7 +165,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         base: &FileName,
         syntax: Syntax,
         module: Option<ModuleConfig>,
-        comments: Option<&'cmt SwcComments>,
+        comments: Option<&'cmt SingleThreadedComments>,
     ) -> impl 'cmt + swc_ecma_visit::Fold
     where
         P: 'cmt,
@@ -180,16 +183,18 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 self.top_level_mark,
                 comments,
                 env,
+                self.assumptions,
             ))
         } else {
+            let assumptions = self.assumptions;
             Either::Right(chain!(
                 Optional::new(
                     compat::es2022::es2022(compat::es2022::Config {
                         class_properties: compat::es2022::class_properties::Config {
-                            private_as_properties: self.loose,
-                            constant_super: self.loose,
-                            set_public_fields: self.loose,
-                            no_document_all: self.loose
+                            private_as_properties: assumptions.private_fields_as_properties,
+                            constant_super: assumptions.constant_super,
+                            set_public_fields: assumptions.set_public_class_fields,
+                            no_document_all: assumptions.no_document_all
                         }
                     }),
                     should_enable(self.target, EsVersion::Es2022)
@@ -201,11 +206,11 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 Optional::new(
                     compat::es2020::es2020(compat::es2020::Config {
                         nullish_coalescing: compat::es2020::nullish_coalescing::Config {
-                            no_document_all: self.loose
+                            no_document_all: assumptions.no_document_all
                         },
                         optional_chaining: compat::es2020::opt_chaining::Config {
-                            no_document_all: self.loose,
-                            pure_getter: self.loose
+                            no_document_all: assumptions.no_document_all,
+                            pure_getter: assumptions.pure_getters
                         }
                     }),
                     should_enable(self.target, EsVersion::Es2020)
@@ -217,8 +222,8 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 Optional::new(
                     compat::es2018(compat::es2018::Config {
                         object_rest_spread: compat::es2018::object_rest_spread::Config {
-                            no_symbol: self.loose,
-                            set_property: self.loose
+                            no_symbol: assumptions.object_rest_no_symbols,
+                            set_property: assumptions.set_spread_properties
                         }
                     }),
                     should_enable(self.target, EsVersion::Es2018)
@@ -237,10 +242,11 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                         comments,
                         compat::es2015::Config {
                             classes: compat::es2015::classes::Config {
-                                constant_super: self.loose,
-                                no_class_calls: self.loose,
-                                set_class_methods: self.loose,
-                                super_is_callable_constructor: self.loose
+                                constant_super: assumptions.constant_super,
+                                no_class_calls: assumptions.no_class_calls,
+                                set_class_methods: assumptions.set_class_methods,
+                                super_is_callable_constructor: assumptions
+                                    .super_is_callable_constructor
                             },
                             computed_props: compat::es2015::computed_props::Config {
                                 loose: self.loose
@@ -254,11 +260,11 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                             },
                             regenerator: self.regenerator,
                             template_literal: compat::es2015::template_literal::Config {
-                                ignore_to_primitive: self.loose,
-                                mutable_template: self.loose
+                                ignore_to_primitive: assumptions.ignore_to_primitive_hint,
+                                mutable_template: assumptions.mutable_template_object
                             },
                             parameters: compat::es2015::parameters::Config {
-                                ignore_function_length: self.loose,
+                                ignore_function_length: assumptions.ignore_function_length,
                             },
                             typescript: syntax.typescript()
                         }
@@ -318,7 +324,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
 struct MinifierPass {
     options: Option<JsMinifyOptions>,
     cm: Lrc<SourceMap>,
-    comments: Option<SwcComments>,
+    comments: Option<SingleThreadedComments>,
     top_level_mark: Mark,
 }
 
