@@ -23,6 +23,7 @@ use swc_cached::regex::CachedRegex;
 pub use swc_common::chain;
 use swc_common::{
     collections::{AHashMap, AHashSet},
+    comments::SingleThreadedComments,
     errors::Handler,
     FileName, Mark, SourceMap, SyntaxContext,
 };
@@ -65,7 +66,7 @@ use self::util::BoolOrObject;
 use crate::{
     builder::PassBuilder,
     plugin::{PluginConfig, PluginContext},
-    SwcComments, SwcImportResolver,
+    SwcImportResolver,
 };
 
 #[cfg(test)]
@@ -257,6 +258,8 @@ impl Default for InputSourceMap {
 
 impl Options {
     /// `parse`: `(syntax, target, is_module)`
+    ///
+    /// `parse` should use `comments`.
     #[allow(clippy::too_many_arguments)]
     pub fn build_as_input<'a, P>(
         &self,
@@ -268,7 +271,7 @@ impl Options {
         handler: &Handler,
         is_module: IsModule,
         config: Option<Config>,
-        comments: Option<&'a SwcComments>,
+        comments: Option<&'a SingleThreadedComments>,
         custom_before_pass: impl FnOnce(&Program) -> P,
     ) -> Result<BuiltInput<impl 'a + swc_ecma_visit::Fold>, Error>
     where
@@ -296,7 +299,7 @@ impl Options {
             ..
         } = config.jsc;
 
-        let assumptions = assumptions.unwrap_or_else(|| {
+        let mut assumptions = assumptions.unwrap_or_else(|| {
             if loose {
                 Assumptions::all()
             } else {
@@ -314,18 +317,21 @@ impl Options {
 
         let mut program = parse(syntax, es_version, is_module)?;
 
+        let mut transform = transform.unwrap_or_default();
+
         // Do a resolver pass before everything.
         //
         // We do this before creating custom passses, so custom passses can use the
         // variable management system based on the syntax contexts.
         if syntax.typescript() {
+            // assumptions.set_class_methods = !transform.use_define_for_class_fields;
+            assumptions.set_public_class_fields = !transform.use_define_for_class_fields;
+
             program.visit_mut_with(&mut ts_resolver(top_level_mark));
         } else {
             program.visit_mut_with(&mut resolver_with_mark(top_level_mark));
         }
 
-        let mut transform = transform.unwrap_or_default();
-
         if program.is_module() {
             js_minify = js_minify.map(|c| {
                 let compress = c
@@ -474,6 +480,7 @@ impl Options {
                             treat_const_enum_as_enum: transform.treat_const_enum_as_enum,
                             ts_enum_is_readonly: assumptions.ts_enum_is_readonly,
                         },
+                        use_define_for_class_fields: !assumptions.set_public_class_fields,
                         ..Default::default()
                     },
                     comments,
@@ -505,6 +512,7 @@ impl Options {
             input_source_map: config.input_source_map.clone(),
             output_path: output_path.map(|v| v.to_path_buf()),
             source_file_name,
+            comments: comments.cloned(),
             preserve_comments,
         })
     }
@@ -966,6 +974,7 @@ pub struct BuiltInput<P: swc_ecma_visit::Fold> {
 
     pub source_file_name: Option<String>,
 
+    pub comments: Option<SingleThreadedComments>,
     pub preserve_comments: Option<BoolOrObject<JsMinifyCommentOption>>,
 
     pub inline_sources_content: bool,
@@ -1181,6 +1190,9 @@ pub struct TransformConfig {
 
     #[serde(default)]
     pub treat_const_enum_as_enum: bool,
+
+    #[serde(default)]
+    pub use_define_for_class_fields: bool,
 }
 
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
